# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-today OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import orm
from openerp.osv import fields
from openerp.tools.translate import _
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import openerp.addons.decimal_precision as dp


class project_job_type(orm.Model):
    """ Job types of project """
    _name = "project.job.type"
    _description = "Job types of project's task, issue, ..."
    _columns = {
                'name': fields.char(
                                    'Name',
                                    size=128,
                                    required=True,
                                    translate=True,
                                    select=True),
                'ref': fields.char(
                                   'Reference',
                                   size=64,
                                   required=True,
                                   select=True),
                'company_id': fields.many2one(
                                              'res.company',
                                              'Company',
                                              required=True,
                                              select=1
                                              ),
                'profitability': fields.boolean(
                                                'Profitability',
                                                help="Specify if the job can be selected for profitability calculation."),
                'periodicity': fields.boolean(
                                              'Periodicity',
                                              help="Specify if the job has a periodic programming"),
                'project_categ_id': fields.many2one(
                                                    'project.category',
                                                    'name',
                                                    required=True,
                                                    help="Category of project's task, issue, ..."
                                                    ),
                'planning': fields.selection([
                                              ('exa', 'Exact Date'),
                                              ('dow', 'Day of Week')
                                              ], 'Planning',
                                             select=True,
                                             required=True,
                                             help=" Exact Date: first working day available  \
                                             from the calculated date. \
                                             \n Day of Week: forcing calculated date day of the week"
                                             ),
                'note': fields.text('Job Documentation'),
                }

    _defaults = {
                 'company_id': lambda self, cr, uid, c: self.pool.get(
                                        'res.company'
                                        )._company_default_get(
                                                               cr,
                                                               uid,
                                                               'project',
                                                               context=c
                                                               ),
                 'profitability': False,
                 'periodicity': False,
                 'planning': 'exa',
                 }

    _order = 'periodicity desc, name asc'


class project_job_month(orm.Model):
    """ Job types of project """
    _name = "project.job.month"
    _description = "Month to Job"
    _columns = {
                'name': fields.char(
                                    'Name',
                                    size=128,
                                    required=True,
                                    translate=True,
                                    select=True),
                'month': fields.integer(
                                       'Month Number',
                                       required=True,
                                       select=True

                                       ),

                }
    _order = 'month asc, id desc'


class project_job_line(orm.Model):

    _name = "project.job.line"
    _description = "Job lines of project"

    def _task_count(self, cr, uid, ids, field_name, arg, context=None):
        if context is None:
            context = {}
        res = dict.fromkeys(ids, 0)
        ctx = context.copy()
        ctx['active_test'] = False
        task_ids = self.pool.get('project.task').search(
                                                        cr,
                                                        uid,
                                                        [(
                                                          'job_line_id', 'in',
                                                          ids
                                                          )],
                                                        context=ctx
                                                        )
        for task in self.pool.get('project.task').browse(
                                                         cr,
                                                         uid,
                                                         task_ids,
                                                         context
                                                         ):
            res[task.job_line_id.id] += 1
        return res

    def _total_hours(self, cr, uid, ids, names, arg, context=None):
        res = {}
        if context is None:
            context = {}
        for job_line in self.browse(cr, uid, ids, context=context):
            hours_task = job_line.hours_task
            if job_line.job_type_id.periodicity == True:
                date_start = datetime.strptime(job_line.project_id.date_start, "%Y-%m-%d")
                new_date = date_start
                date_stop = datetime.strptime(job_line.project_id.date, "%Y-%m-%d")
                prolong_interval = job_line.prolong_interval
                prolong_unit = job_line.prolong_unit
                dayofweek = job_line.dayofweek
                planning = job_line.job_type_id.planning,
                if date_start and date_stop and prolong_interval >= 1 and prolong_unit and hours_task != 0:
                    hours = 0
                    task_dates = []
                while new_date < date_stop:
                    new_date = self.__get_next_term_date(
                                                         new_date,
                                                         prolong_unit,
                                                         prolong_interval
                                                         )
                    planning_date = self.__get_next_day_week(
                                                    new_date,
                                                    planning,
                                                    dayofweek,
                                                    prolong_unit,
                                                    prolong_interval
                                                         )
                    task_dates.append(planning_date)
                if len(task_dates) > 0:
                    hours = hours_task * len(set(task_dates))
                    res[job_line.id] = hours
            else:
                if len(job_line.month_ids) > 0:
                    hours = hours_task * len(job_line.month_ids)
                    res[job_line.id] = hours
        return res

    _columns = {

                'name': fields.char(
                                    'Job Name',
                                    size=64,
                                    required=True,
                                    translate=True
                                    ),

                'prolong_interval': fields.integer(
                                                   'Interval',
                                                   help="* Interval in time units to prolong the agreement until new renewall \
                                                   (that is automatic for unlimited term, manual for renewable fixed term). "
                                                   ),
                'prolong_unit': fields.selection([
                                                  ('days', 'days'),
                                                  ('weeks', 'weeks'),
                                                  ('months', 'months'),
                                                  ('years', 'years')],
                                                 'Interval unit',
                                                 help='Time unit for the prolongation interval'
                                                 ),
                'job_type_id': fields.many2one(
                                           'project.job.type',
                                           'type_id',
                                           'Job Type',
                                           ondelete='cascade',
                                           select=True,
                                           required=True
                                           ),
                'project_id': fields.many2one(
                                              'project.project',
                                              'Project',
                                              help="Link this job line to a Project",
                                              required=True,
                                              select=True,
                                              ondelete='cascade',
                                              readonly=True
                                              ),
                'analytic_account_id': fields.related(
                                                    'project_id',
                                                    'analytic_account_id',
                                                    type='many2one',
                                                    relation='account.analytic.account',
                                                    string="Analityc Account",
                                                    store=True
                                                      ),
                'tasks': fields.one2many(
                                         'project.task',
                                         'job_line_id',
                                         "Task Activities"
                                         ),
                'task_count': fields.function(
                                              _task_count,
                                              type='integer',
                                              string="Open Tasks"
                                              ),

                'hours': fields.function(_total_hours,
                                      string="Year Hours",
                                      help="Total job time for year.",
                                      required=False,
                                      store=True
                                      ),
                'hours_task': fields.float(
                                      'Task Hours',
                                      digits=(16, 2),
                                      required=True,
                                      help="Time for Task.",
                                      ),
                'dayofweek': fields.selection([
                                               ('0', 'Monday'),
                                               ('1', 'Tuesday'),
                                               ('2', 'Wednesday'),
                                               ('3', 'Thursday'),
                                               ('4', 'Friday'),
                                               ('5', 'Saturday'),
                                               ('6', 'Sunday')
                                               ], 'Day of Week',
                                               select=True,
                                               required=False,
                                               ),
                'notes': fields.text('Notes'),
                'profitability': fields.boolean(
                                                'Profitability',
                                                help="Specify if the job can be selected for profitability calculation."
                                                ),
                'periodicity': fields.boolean(
                                              'Periodicity',
                                              help="Specify if the job has a periodic programming"
                                              ),
                'month_ids': fields.many2many(
                                         'project.job.month',
                                         id1='line_id',
                                         id2='month_id',
                                         string='Month',
                                         )

                }

    _defaults = {
                'profitability': True,
                'periodicity': True,
                'dayofweek': '0',
                'prolong_interval': 1,
                'hours_task': 1,
                'prolong_unit': 'years',
                }

    _order = 'periodicity desc, job_type_id asc'

    def __get_next_term_date(self, date, unit, interval):
        """
        Get the date that results on incrementing given date an interval
        of time in time unit.
        @param date: Original date.
        @param unit: Interval time unit.
        @param interval: Quantity of the time unit.
        @rtype: date
        @return: The date incremented in 'interval' units of 'unit'.
        """
        if unit == 'days':
            return date + timedelta(days=interval)
        elif unit == 'weeks':
            return date + timedelta(weeks=interval)
        elif unit == 'months':
            return date + relativedelta(months=interval)
        elif unit == 'years':
            return date + relativedelta(years=interval)

    def __get_next_day_week(
                            self,
                            prog_date,
                            planning, dayofweek,
                            prolong_unit,
                            prolong_interval
                            ):
        """
        Get the date that results on incrementing given date an interval
        of time in time unit.
        @param date: Original date.
        @param planning: Planning type exa= exact or dow= day of week
        @param dayofweek: Day of Week in number.
        @rtype: date
        @return: The date for type planning and day of week.
        """
        n_dayofweek = int(dayofweek) + 1
        week_day = prog_date.weekday() + 1
        if planning == "exa":
            if week_day == 6:
                return prog_date + timedelta(days=2)
            elif week_day == 7:
                return prog_date + timedelta(days=1)
            else:
                return prog_date
        else:
            if prolong_unit != 'days':
                if n_dayofweek > week_day:
                    return prog_date - timedelta(days=(week_day - n_dayofweek))
                elif n_dayofweek < week_day:
                    return prog_date + timedelta(days=(n_dayofweek - week_day))
                elif week_day == n_dayofweek:
                    return prog_date
            else:
                if week_day >= 6:
                    if prolong_interval == 1:
                        if week_day == 6:
                            return prog_date - timedelta(days=1)
                        else:
                            return prog_date - timedelta(days=2)
                    return prog_date - timedelta(days=prolong_interval)
                return prog_date

    def onchange_job_type(self, cr, uid, ids, job_type_id, date_start, date_stop=False, context=None):
        context = context or {}
        if not job_type_id:
            return {}
        if not  date_stop:
            raise orm.except_orm(
                                 _('No Date End Defined!'),
                                 _('Before choosing a job,\n please input date end for this project.')
                                 )
        job_type = self.pool.get(
                                 'project.job.type'
                                 ).browse(
                                          cr,
                                          uid,
                                          job_type_id,
                                          context=context
                                          )
        values = {}
        values.update({
                       'profitability': job_type.profitability,
                       'periodicity': job_type.periodicity,
                       'name': job_type.name
                       })
        result = {'value': values}
        return result


class project_project(orm.Model):

    _inherit = "project.project"
    _columns = {

                'job_line': fields.one2many(
                                           'project.job.line',
                                           'project_id',
                                           'Job Lines',
                                           ondelete='cascade'
                                           ),
                }


class task(orm.Model):

    _inherit = "project.task"

    def _num_week(self, cr, uid, ids, field_names, args, context=None):
        if context is None:
            context = {}
        res = {}
        for task in self.browse(cr, uid, ids, context=context):
            if task.date_start:
                date_start = datetime.strptime(
                                           task.date_start,
                                           "%Y-%m-%d %H:%M:%S"
                                           )
                Year = date_start.year
                Week = date_start.isocalendar()[1]
                res[task.id] = str(Year) + '-' + str(Week).zfill(2)
        return res

    _columns = {
                'job_type_id': fields.many2one(
                                           'project.job.type',
                                           'type_id',
                                           'Job Type',
                                           ondelete='cascade',
                                           select=True,
                                           required=True
                                           ),
                'week': fields.function(
                                        _num_week,
                                        string='Week',
                                        help="Number week of Year",
                                        type='char',
                                        size=9,
                                        store=True
                                        ),
                'job_line_id': fields.many2one(
                                               'project.job.line',
                                               'line_id',
                                               'Job Line',
                                               select=True,
                                               required=False
                                               ),
                }

    def action_close_job(self, cr, uid, ids, context=None):
        """ This action closes the task
        """
        task_id = len(ids) and ids[0] or False
        self._check_child_task(cr, uid, ids, context=context)
        if task_id:
            task = self.pool.get('project.task').browse(cr, uid, task_id, context=context)
            if task.user_id.id:
                res = {
                       'name': task.name,
                       'date': task.date_start,
                       'task_id': task.id,
                       'hours': task.planned_hours,
                       'user_id': task.user_id.id
                       }
            else:
                raise orm.except_orm(
                                     _('Error in Task'),
                                     _('User has not been assigned to this task')
                                     )
            self.pool.get('project.task.work').create(cr, uid, res, context=context)

        else:
            return False

        return self.do_close(cr, uid, [task_id], context=context)




# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
