# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-today OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import orm
from openerp.osv import fields
from openerp.tools.translate import _
from datetime import datetime, timedelta, date, time
from dateutil.relativedelta import relativedelta
import time


class project_make_task_line (orm.TransientModel):
    _name = "project.make.task.line"
    _description = "Project Make Task Lines"
    _columns = {
                'project_id': fields.many2one(
                                              'project.project',
                                              'proyect_id',
                                              'Project',
                                              ondelete='cascade',
                                              required=True
                                              ),
                'job_line_id': fields.many2one(
                                           'project.job.line',
                                           'line_id',
                                           'Job Lines',
                                           ondelete='cascade',
                                           required=True
                                           ),
                'hours': fields.float(
                                      'Year Hours',
                                      digits=(16, 2),
                                      help="Total job time for year.",
                                      required=True
                                      ),
                'hours_task': fields.float(
                                      'Task Hours',
                                      digits=(16, 2),
                                      help="Time for Task.",
                                      required=True
                                      ),
                'dayofweek': fields.selection([
                                               ('0', 'Monday'),
                                               ('1', 'Tuesday'),
                                               ('2', 'Wednesday'),
                                               ('3', 'Thursday'),
                                               ('4', 'Friday'),
                                               ('5', 'Saturday'),
                                               ('6', 'Sunday')
                                               ], 'Day of Week',
                                               select=True,
                                               required=True,
                                               ),
                'prolong_interval': fields.integer(
                                                   'Interval',
                                                   required=True,
                                                   help="Interval in time units to prolong the agreement until new renewall (that is automatic for unlimited term, manual for renewable fixed term)."),

                'prolong_unit': fields.selection([
                                                  ('days', 'days'),
                                                  ('weeks', 'weeks'),
                                                  ('months', 'months'),
                                                  ('years', 'years')],
                                                 'Interval unit',
                                                 required=True,
                                                 help='Time unit for the prolongation interval'),
                'user_id': fields.many2one(
                                           'res.users',
                                           'Assigned to',
                                           select=True,
                                           required=False
                                           ),
                'date_start': fields.datetime(
                                              'Starting Date',
                                              select=True,
                                              required=True
                                              ),
                'date_end': fields.datetime(
                                            'Ending Date',
                                            select=True,
                                            required=True
                                            ),
                'wizard_id': fields.many2one(
                                             'project.make.task',
                                             string="Wizard",
                                             ondelete='cascade'
                                             ),
                }
    _defaults = {
                'date_start': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
                'date_end': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S')
                }


class project_make_task (orm.TransientModel):
    _name = "project.make.task"
    _description = "Project Make Task"
    _columns = {
                'jobs_ids': fields.one2many(
                                            'project.make.task.line',
                                            'wizard_id',
                                            'Job Line'
                                            ),
                'sporadic_job': fields.boolean(
                                              'Programming sporadic Jobs',
                                              help=" Check for programming sporadic Jobs for project in list"
                                               )
                }

    _defaults = {
                'sporadic_job': True,
                }

    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        res = super(project_make_task, self).default_get(cr, uid, fields, context=context)
        project_ids = context.get('active_ids', [])
        active_model = context.get('active_model')

        if not project_ids:
            # si no se selecciona proyecto(s)
            return res
        assert active_model in ('project.project'), 'Bad context propagation'
        if 'jobs_ids' in fields:
            projects = self.pool.get(
                                     'project.project'
                                     ).browse(cr,
                                              uid,
                                              project_ids,
                                              context=context
                                              )
            lines = []
            for project in projects:
                for j in project.job_line:
                    if j.periodicity:
                        jobs = (self._jobs_for(cr, uid, j))
                        lines.append(jobs)
            res.update(jobs_ids=lines)
        return res

    def _jobs_for(self, cr, uid, job):
        jobs = {
                'project_id': job.project_id.id,
                'job_line_id': job.id,
                'date_start': job.project_id.date_start,
                'date_end': job.project_id.date,
                'hours': job.hours,
                'dayofweek': job.dayofweek,
                'prolong_interval': job.prolong_interval,
                'prolong_unit': job.prolong_unit,
                'hours_task': job.hours_task
                }
        return jobs

    def make_task(self, cr, uid, ids, context=None):
        task_wizard = self.browse(cr, uid, ids[0], context=context)
        total_hours = 0
        project_obj = self.pool.get('project.project')
        task_obj = self.pool.get('project.task')
        project_stage_obj = self.pool.get('project.task.type')
        job_line_obj = self.pool.get('project.job.line')
        #Check UTC user_id
        if context.get('tz') != 'Europe/Madrid':
            raise orm.except_orm(
                                 _('TimeZone Error !'),
                                 _('Please select the timezone Europe/Madrid')
                                 )
        #Check user_id in job lines
        for job_check in task_wizard.jobs_ids:
            if not job_check.user_id:
                raise orm.except_orm(
                                 _('No User Defined!'),
                                 _('Please select an employee for this job. \n Project: "%s" \n Job: "%s"') % (job_check.project_id.name, job_check.job_line_id.name)
                                 )

        for job in task_wizard.jobs_ids:
            #Check stage_id
            task_dates = []
            project_id = job.project_id.id
            stage_id = project_stage_obj.search(cr, uid, [
                                                    ('state', '=', 'pending'),
                                                    ('project_ids', '=', project_id)
                                                          ]
                                                )
            date_start = datetime.strptime(job.date_start, "%Y-%m-%d %H:%M:%S")
            date_stop = datetime.strptime(job.date_end, "%Y-%m-%d %H:%M:%S")
            while date_start < date_stop:
                date_start = self.__get_next_term_date(
                                                       date_start,
                                                       job.prolong_unit,
                                                       job.prolong_interval
                                                       )
                planning_date = self.__get_next_day_week(
                                                    date_start,
                                                    job.job_line_id.job_type_id.planning,
                                                    job.dayofweek,
                                                    job.prolong_unit,
                                                    job.prolong_interval
                                                         )
                # Date in Timezone Info and check is STD o DST
                tzplanning_date = fields.datetime.context_timestamp(
                                                                cr,
                                                                uid,
                                                                planning_date,
                                                                context=context
                                                                    )
                tz_seconds = tzplanning_date.tzinfo._utcoffset.seconds
                if tz_seconds == 7200:
                    task_dates.append(planning_date)
                else:
                    task_dates.append(planning_date + timedelta(seconds=3600))
            #Unique dates in list
            task_dates = set(task_dates)
            #Calculated field for task and create this
            for task_date_start in task_dates:
                # Add hours task to date_end
                task_date_end = task_date_start + timedelta(
                                                        hours=job.hours_task
                                                            )
                categ_ids = [job.job_line_id.job_type_id.project_categ_id.id]
                res = {
                       'name': job.job_line_id.name,
                       'job_line_id': job.job_line_id.id,
                       'description': job.job_line_id.notes,
                       'date_start': task_date_start,
                       'date_end': task_date_end,
                       'date_deadline': task_date_end,
                       'project_id': project_id,
                       'planned_hours': job.hours_task,
                       'user_id': job.user_id.id,
                       'manager_id': job.project_id.analytic_account_id.user_id.id,
                       'job_type_id': job.job_line_id.job_type_id.id,
                       'company_id': job.project_id.company_id.id,
                       'partner_id': job.project_id.partner_id.id,
                       'categ_ids': [(6, 0, categ_ids)],
                       'remaining_hours': job.hours_task,
                       'state': 'pending',
                       'stage_id': stage_id[0]
                       }
                task_obj.create(cr, uid, res, context=context)
                # Update total_hours and planned_hours from project
                total_hours = total_hours + job.hours_task
                res.clear()
            prj_total_hours = float(project_obj.read(cr, uid, project_id, ['total_hours'], context=context)['total_hours'])
            prj_planned_hours = float(project_obj.read(cr, uid, project_id, ['planned_hours'], context=context)['planned_hours'])
            res = {
                   'total_hours': total_hours + prj_total_hours,
                   'planned_hours': total_hours + prj_planned_hours
                   }
            project_obj.write(cr, uid, [project_id], res, context=context)
        #Check if programming sporadic jobs
        if task_wizard.sporadic_job == True:
            project_ids = []
            project_id = 0
            task_month = []
            task_month_dates = []
            #Check project in job line and extract unique ids
            for job in task_wizard.jobs_ids:
                project_ids.append(job.project_id.id)
            project_ids = set(project_ids)

            for project_id in project_ids:
                #Search date start and date stop in project
                task_month = []
                date_start = datetime.strptime(
                                               project_obj.read(
                                                                cr,
                                                                uid,
                                                                project_id,
                                                                ['date_start'],
                                                                context=context
                                                                )['date_start'], "%Y-%m-%d"
                                               )

                date_stop = datetime.strptime(
                                               project_obj.read(
                                                                cr,
                                                                uid,
                                                                project_id,
                                                                ['date'],
                                                                context=context
                                                                )['date'], "%Y-%m-%d"
                                               )
                # Calculate month for this project
                while date_start < date_stop:
                    date_start = self.__get_next_term_date(
                                                       date_start,
                                                       'months',
                                                       1,
                                                       )
                    task_month.append(date_start)
                #search job lines
                esporadic_job_ids = job_line_obj.search(
                                                        cr,
                                                        uid,
                                                        [('project_id', '=', project_id),
                                                         ('periodicity', '=', False)]
                                                        )
                # If esporadic jobs find dates
                if esporadic_job_ids:
                    for sporadic_id in esporadic_job_ids:
                        task_month_dates=[]
                        job_line = job_line_obj.browse(cr, uid, sporadic_id, context=context)
                        for month in task_month:
                            for line_month in job_line.month_ids:
                                    if line_month.id == month.month:
                                        # Date in Timezone Info and check is STD o DST
                                        tzplanning_date = fields.datetime.context_timestamp(
                                                                    cr,
                                                                    uid,
                                                                    month,
                                                                    context=context
                                                                    )
                                        tz_seconds = tzplanning_date.tzinfo._utcoffset.seconds
                                        if tz_seconds == 7200:
                                            task_month_dates.append(month)
                                        else:
                                            task_month_dates.append(month + timedelta(seconds=3600))
                        #Prepare create tasks
                        stage_id = project_stage_obj.search(
                                                            cr,
                                                            uid,
                                                            [('state', '=', 'pending'),
                                                             ('project_ids', '=', project_id)]
                                                            )

                        for task_month_date in task_month_dates:
                            #Aqui tengo que ver si es el primer dia del mes
                            if task_month_date.day > 1:
                                task_month_date - timedelta(
                                                            days=task_month_date.day - (task_month_date.day - 1)
                                                            )
                            task_date_start = task_month_date + timedelta(
                                                                          hours=6
                                                                          )
                            task_date_end = task_date_start + timedelta(
                                                                        hours=job.hours_task
                                                                        )
                            res = {
                               'name': job_line.name,
                               'job_line_id': job_line.id,
                               'description': job_line.notes,
                               'date_start': task_date_start,
                               'date_end': task_date_end,
                               'date_deadline': task_date_end,
                               'project_id': project_id,
                               'planned_hours': job.hours_task,
                               'manager_id': job.project_id.analytic_account_id.user_id.id,
                               'job_type_id': job_line.job_type_id.id,
                               'company_id': job_line.project_id.company_id.id,
                               'partner_id': job_line.project_id.partner_id.id,
                               'user_id': '',
                               'categ_ids': [(6, 0, categ_ids)],
                               'remaining_hours': job_line.hours_task,
                               'state': 'pending',
                               'stage_id': stage_id[0]
                               }
                            task_obj.create(cr, uid, res, context=context)
                            # Update total_hours and planned_hours from project
                            total_hours = total_hours + job.hours_task
                            res.clear()
            prj_total_hours = float(project_obj.read(cr, uid, project_id, ['total_hours'], context=context)['total_hours'])
            prj_planned_hours = float(project_obj.read(cr, uid, project_id, ['planned_hours'], context=context)['planned_hours'])
            res = {
                   'total_hours': total_hours + prj_total_hours,
                   'planned_hours': total_hours + prj_planned_hours
                   }
            project_obj.write(cr, uid, [project_id], res, context=context)

        return {'type': 'ir.actions.act_window_close'}

    def __get_next_term_date(self, date, unit, interval):
        """
        Get the date that results on incrementing given date an interval
        of time in time unit.
        @param date: Original date.
        @param unit: Interval time unit.
        @param interval: Quantity of the time unit.
        @rtype: date
        @return: The date incremented in 'interval' units of 'unit'.
        """
        if unit == 'days':
            return date + timedelta(days=interval)
        elif unit == 'weeks':
            return date + timedelta(weeks=interval)
        elif unit == 'months':
            return date + relativedelta(months=interval)
        elif unit == 'years':
            return date + relativedelta(years=interval)

    def __get_next_day_week(
                            self,
                            prog_date,
                            planning, dayofweek,
                            prolong_unit,
                            prolong_interval
                            ):
        """
        Get the date that results on incrementing given date an interval
        of time in time unit.
        @param date: Original date.
        @param planning: Planning type exa= exact or dow= day of week
        @param dayofweek: Day of Week in number.
        @rtype: date
        @return: The date for type planning and day of week.
        """
        n_dayofweek = int(dayofweek) + 1
        week_day = prog_date.weekday() + 1
        if planning == "exa":
            if week_day == 6:
                return prog_date + timedelta(days=2)
            elif week_day == 7:
                return prog_date + timedelta(days=1)
            else:
                return prog_date
        else:
            if prolong_unit != 'days':
                if n_dayofweek > week_day:
                    return prog_date - timedelta(days=(week_day - n_dayofweek))
                elif n_dayofweek < week_day:
                    return prog_date + timedelta(days=(n_dayofweek - week_day))
                elif week_day == n_dayofweek:
                    return prog_date
            else:
                if week_day >= 6:
                    if prolong_interval == 1:
                        if week_day == 6:
                            return prog_date - timedelta(days=1)
                        else:
                            return prog_date - timedelta(days=2)
                    return prog_date - timedelta(days=prolong_interval)
                return prog_date


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
